@@ -2,7 +2,8 @@
 # monthlyfunction: Generic function for applying any R function to     #
 #                  ALL the values in 'x' belonging to a given month    #
 ########################################################################
-#                  May 15th, 2009; Aug 31th 2009                       #
+# Started: May 15th, 2009;                                             #
+# Updates: 31-Aug-2009 ; 25-Jul-2011 ; 08-Aug-2011                     #
 ########################################################################
 # 'x   '    : variable of type 'zoo' or 'data.frame', with daily or monthly frequency
 # 'FUN'      : Function that will be applied to ALL the values in 'x' belonging to each one of the 12 months of the year
@@ -13,44 +14,67 @@ monthlyfunction <- function(x, ...) UseMethod("monthlyfunction")
 
 monthlyfunction.default <- function(x, FUN, na.rm=TRUE,...) {
 
-     # Checking that 'x' is a zoo object
-     if (is.na(match(class(x), c("zoo"))))
-	stop("Invalid argument: 'x' must be of class 'zoo'")
-
-     # Checking that the user provied a valid argument for 'FUN'
-     if (missing(FUN))
-         stop("Missing argument: 'FUN' must be provided")
-
-     # Checking the user provide a valid value for 'x'
-     if (is.na(match(sfreq(x), c("daily", "monthly"))))
-	stop(paste("Invalid argument: 'x' is not a daily or mothly ts, it is a ", sfreq(x), " ts", sep="") )
+     # Checking that the user provied a valid class for 'x'   
+     valid.class <- c("xts", "zoo")    
+     if (length(which(!is.na(match(class(x), valid.class )))) <= 0)  
+        stop("Invalid argument: 'class(x)' must be in c('xts', 'zoo')")
 
      # Requiring the Zoo Library (Z’s ordered observations)
      require(zoo)
 
-     # 'as.numeric' is necessary for being able to change the names to the output
-     totals <- as.numeric( aggregate( x, by= format( time(x), "%m" ), FUN=FUN, na.rm= na.rm ) )
-
-     # Replacing the NaNs by 'NA.
-     # NaN's are obtained when using theFUN=mean with complete NA values
-     nan.index          <- which(is.nan(totals))
-     totals[ nan.index] <- NA
-
-     # Getting the position of all the years in which there were no values
-     # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
-     inf.index <- which(is.infinite(totals))
-
-     # Changing all the Inf and -Inf by NA's
-     if ( length(inf.index) > 0 ) { totals[inf.index] <- NA }
-
-     names(totals) <- month.abb
-
-     return(totals)
+     monthlyfunction.zoo(x=x, FUN=FUN, na.rm=na.rm,...)
 
 } # 'monthlyfunction.default' end
 
 
+########################################
+# Author : Mauricio Zambrano-Bigiarini #
+# Started: 25-Jul-2011                 #
+# Updates: 08-Aug-2011                 #
+########################################
+monthlyfunction.zoo <- function(x, FUN, na.rm=TRUE,...) {
 
+     # Checking that the user provied a valid argument for 'FUN'
+     if (missing(FUN))  stop("Missing argument: 'FUN' must be provided")
+     
+     # Checking the user provide a valid value for 'x'
+     if (is.na(match(sfreq(x), c("daily", "monthly"))))
+	stop(paste("Invalid argument: 'x' is not a daily or mothly ts, it is a ", sfreq(x), " ts", sep="") )
+
+     # Monthly index for 'x'
+     dates  <- time(x)
+     m      <- as.numeric(format( dates, "%m" ))
+     months <- factor( month.abb[m], levels=unique(month.abb[m]) )
+     
+     # 'as.numeric' is necessary for being able to change the names to the output
+     totals <- aggregate(x, by= months, FUN=FUN, na.rm= na.rm )
+
+     # Replacing the NaNs by 'NA.
+     # NaN's are obtained when using the FUN=mean with complete NA values
+     nan.index          <- which(is.nan(totals))
+     if ( length(nan.index) > 0 )  totals[ nan.index] <- NA
+     
+     # Replacing all the Inf and -Inf by NA's
+     # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
+     inf.index <- which(is.infinite(totals))
+     if ( length(inf.index) > 0 ) totals[inf.index] <- NA
+
+     # Giving meaningful names to the output
+     if ( (is.matrix(x)) | (is.data.frame(x)) ) {
+       totals <- t(totals) # For having the months' names as column names
+     } #IF end
+
+     return(totals)
+
+} # 'monthlyfunction.zoo' end
+ 
+ 
+
+########################################
+# Author : Mauricio Zambrano-Bigiarini #
+# Started: 25-Jul-2011                 #
+# Updates: 08-Aug-2011                 #
+######################################## 
 # 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
 #             dates for correponding to the 'sname' station
 #             If 'dates' is a number, it indicates the index of the column in
@@ -105,105 +129,104 @@ monthlyfunction.data.frame <- function(x, FUN, na.rm=TRUE,
     tmp   <- dates
     dates <- as.Date(x[, dates], format= date.fmt)
     x     <- x[-tmp]
-  }  # IF end
-
-  # If 'dates' is a factor, it have to be converted into 'Date' class,
-  # using the date format  specified by 'date.fmt'
-  if ( class(dates) == "factor" ) dates <- as.Date(dates, format= date.fmt)
-
-  # Amount of stations in 'x'
-  nstations <- ncol(x)
-
-  # ID of all the stations in 'x'
-  snames <- colnames(x)
-
-  # Computing the Starting and Ending Year of the analysis
-  Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
-  Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))
-
-  # Amount of Years belonging to the desired period
-  nyears <- Ending.Year - Starting.Year + 1
-
-  # Amount of months belonging to the desired period
-  nmonths <- 12*nyears
-
-  # Requiring the Zoo Library (Z’s ordered observations)
-  require(zoo)
-
-  if (verbose) message("[Starting the computations...]")
+  }  else
+      # If 'dates' is a factor, it have to be converted into 'Date' class,
+      # using the date format  specified by 'date.fmt'
+      if ( class(dates) == "factor" ) {
+	    dates <- zoo::as.Date(dates, format= date.fmt)
+	  } else
+	    # If 'dates' is already of Date class, the following line verifies that
+            # the number of days in 'dates' be equal to the number of element in the
+            # time series corresponding to the 'st.name' station
+            if ( ( class(dates) == "Date") & (length(dates) != nrow(x) ) )
+              stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")
 
+  # Transforming 'x' into a zoo object
+  x <- zoo(x, dates)
+  
+  ##############################################################################
   if (out.type == "data.frame") {
-
-	# Vector with the names of the field that will be used for storing the results
-	field.names <- month.abb
-
-	# Creating the data.frame that will store the computed averages for each station
-	z <- as.data.frame(matrix(data = NA, nrow = nstations, ncol = 12,
-						byrow = TRUE, dimnames = NULL) )
-
-	z <- sapply(1:nstations, function(j,y) {
-
-
-		if (verbose) message( paste("Station: ", format(snames[j], width=10, justify="left"),
-					    " : ",format(j, width=3, justify="left"), "/",
-					    nstations, " => ",
-					    format(round(100*j/nstations,2), width=6, justify="left"),
-					    "%", sep="") )
-
-		# Transforming the column of 'x' into a zoo object,
-		# using the dates provided by the user
-		tmp <- vector2zoo(x=x[,j], dates=dates, date.fmt=date.fmt)
-
-		# Computing the annual values
-		z[,1:12] <- monthlyfunction.default(x= tmp, FUN=FUN, na.rm=na.rm)
-
-	}, y = x) # sapply END
-
-    z <- t(z) # I don't know WHY !!
-    rownames(z) <- snames
-
+  
+    monthlyfunction.zoo(x=x, FUN=FUN, na.rm=na.rm, ...)
+    
   } else if (out.type == "db") {
+  
+        # Amount of stations in 'x'
+        nstations <- ncol(x)
+        
+        # ID of all the stations in 'x'
+        snames <- colnames(x)
+      
+        if (is.null(snames)) snames <- paste("V", 1:nstations, sep="")
+  
+        # Computing the Starting and Ending Year of the analysis
+        Starting.Year <- as.numeric(format(start(x), "%Y"))
+        Ending.Year   <- as.numeric(format(end(x), "%Y"))
+
+        # Amount of Years belonging to the desired period
+        nyears <- Ending.Year - Starting.Year + 1
+        
+        # Computing the numeric index of the resulting months
+        month.index <- unique(as.numeric(format( time(x), "%m" )))
+     
+        # Amount of different months belonging to the desired period
+        nmonths <- length(month.index)
+        
+        # Total amount of months belonging to the desired period
+        totalmonths <- nmonths*nyears
 
         # Creating a vector with the names of the field that will be used for storing the results
         field.names <- c("StationID", "Year", "Month", "Value" )
 
         # Creating the data.frame that will store the computed averages for each subcatchment
-        z <- as.data.frame(matrix(data = NA, nrow = nmonths*nstations, ncol = 4,
-                           byrow = TRUE, dimnames = NULL) )
-
-        y = x
-
+        z <- as.data.frame(matrix(data = NA, nrow = totalmonths*nstations, ncol = 4, byrow = TRUE, dimnames = NULL) )
+        
         for (j in 1:nstations) {
 
-            if (verbose) message( paste("Station: ", format(snames[j], width=10, justify="left"),
+            if (verbose) message( paste("[ Station: ", format(snames[j], width=10, justify="left"),
                                         " : ", format(j, width=3, justify="left"), "/",
                                         nstations, " => ",
                                         format(round(100*j/nstations,2), width=6, justify="left"),
-                                        "%", sep="") )
-
-            # Transforming the column of 'x' into a zoo object,
-	    # using the dates provided by the user
-	    tmp <- vector2zoo(x=x[,j], dates=dates, date.fmt=date.fmt)
+                                        "% ]", sep="") )
 
 	    # Computing the annual values
-	    tmp <- monthlyfunction.default(x= tmp, FUN=FUN, na.rm=na.rm)
+	    tmp <- monthlyfunction.default(x= x[,j], FUN=FUN, na.rm=na.rm, ...)
 
 	    # Putting the annual/monthly values in the output data.frame
             # The first column of 'x' corresponds to the Year
-            row.ini <- (j-1)*nmonths + 1
-            row.fin <-  j*nmonths
+            row.ini <- (j-1)*totalmonths + 1
+            row.fin <-  j*totalmonths
 
-            z[row.ini:row.fin, 1] <- snames[j] # it is automatically repeted 'nmonths' times
-            z[row.ini:row.fin, 2] <- rep(Starting.Year:Ending.Year, each=12)
-            z[row.ini:row.fin, 3] <- month.abb
+            z[row.ini:row.fin, 1] <- snames[j] # it is automatically repeted 'totalmonths' times
+            z[row.ini:row.fin, 2] <- rep(Starting.Year:Ending.Year, each=nmonths)
+            z[row.ini:row.fin, 3] <- month.abb[month.index]
             z[row.ini:row.fin, 4] <- tmp
 
         } # FOR end
-
+        
         colnames(z) <- field.names
-
-    } # IF end
-
-  return( z )
+        
+        return( z )
+      
+    } # ELSE end
 
  } #'monthlyfunction.data.frame' END
+ 
+ 
+########################################
+# Author : Mauricio Zambrano-Bigiarini #
+# Started: 25-Jul-2011                 #
+# Updates: 08-Aug-2011                 #
+########################################
+monthlyfunction.matrix <- function(x, FUN, na.rm=TRUE,
+                                   dates, date.fmt="%Y-%m-%d",
+                                   out.type="data.frame",
+                                   verbose=TRUE,...) {
+ x <- as.data.frame(x)
+ #NextMethod("daily2annual")
+ monthlyfunction.data.frame(x=x, FUN=FUN, na.rm=na.rm,
+                            dates=dates, date.fmt=date.fmt,
+                            out.type=out.type,
+                            verbose=verbose,...)
+                                                                    
+ } # 'monthlyfunction.matrix' END  
